<!DOCTYPE html>
<html lang="{{'{{'}}request.locale_name{{'}}'}}">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="description" content="pyramid web application">
    <meta name="author" content="Camptocamp">

    <title>{{cookiecutter.project}}</title>
    <link rel="shortcut icon" href="{{'{{'}}request.static_url('{{cookiecutter.package}}:static/favicon.ico'){{'}}'}}">

    <!-- Bootstrap core CSS -->
    <link href="{{'{{'}}request.static_url('{{cookiecutter.package}}:node_modules/bootstrap/dist/css/bootstrap.min.css'){{'}}'}}" rel="stylesheet">
    <link href="{{'{{'}}request.static_url('{{cookiecutter.package}}:node_modules/bootstrap-table/dist/bootstrap-table.min.css'){{'}}'}}" rel="stylesheet">
    <link href="{{'{{'}}request.static_url('{{cookiecutter.package}}:node_modules/jquery-steps/demo/css/jquery.steps.css'){{'}}'}}" rel="stylesheet">
    <link href="{{'{{'}}request.static_url('c2cgeoform:static/theme.css'){{'}}'}}" rel="stylesheet">
{% raw %}
    <!-- Deform dependencies CSS -->
    {% if deform_dependencies %}
    <link rel="stylesheet" href="{{'{{'}}request.static_url('deform:static/css/form.css'){{'}}'}}" type="text/css" />
    {% for css in deform_dependencies['css'] %}
    <link rel="stylesheet" type="text/css" href="{{'{{'}}request.static_url(css){{'}}'}}"/>
    {% endfor %}
    {% endif %}
{% endraw %}
    <!-- Custom styles for this scaffold -->
    <link href="{{'{{'}}request.static_url('{{cookiecutter.package}}:static/theme.css'){{'}}'}}" rel="stylesheet">

    <!-- HTML5 shim and Respond.js IE8 support of HTML5 elements and media queries -->
    <!--[if lt IE 9]>
      <script src="//oss.maxcdn.com/libs/html5shiv/3.7.0/html5shiv.js" integrity="sha384-0s5Pv64cNZJieYFkXYOTId2HMA2Lfb6q2nAcx2n0RTLUnCAoTTsS0nKEO27XyKcY" crossorigin="anonymous"></script>
      <script src="//oss.maxcdn.com/libs/respond.js/1.3.0/respond.min.js" integrity="sha384-f1r2UzjsxZ9T4V1f2zBO/evUqSEOpeaUUZcMTz1Up63bl4ruYnFYeM+BxI4NhyI0" crossorigin="anonymous"></script>
    <![endif]-->

    <!-- Bootstrap core JavaScript -->
    <script src="{{'{{'}}request.static_url('{{cookiecutter.package}}:node_modules/jquery/dist/jquery.min.js'){{'}}'}}"></script>
    <script src="{{'{{'}}request.static_url('{{cookiecutter.package}}:node_modules/bootstrap/dist/js/bootstrap.min.js'){{'}}'}}"></script>
    <script src="{{'{{'}}request.static_url('{{cookiecutter.package}}:node_modules/bootstrap-table/dist/bootstrap-table.min.js'){{'}}'}}"></script>
    <script src="{{'{{'}}request.static_url('{{cookiecutter.package}}:node_modules/bootstrap-table/dist/bootstrap-table-locale-all.js'){{'}}'}}"></script>
    <script src="{{'{{'}}request.static_url('{{cookiecutter.package}}:node_modules/jquery.scrollintoview/jquery.scrollintoview.js'){{'}}'}}"></script>
    <script src="{{'{{'}}request.static_url('{{cookiecutter.package}}:node_modules/jquery-steps/build/jquery.steps.min.js'){{'}}'}}"></script>

    <!-- c2cgeoform  -->
    <script src="{{'{{'}}request.static_url('c2cgeoform:static/dist/index.js'){{'}}'}}"></script>
    <link href="{{'{{'}}request.static_url('c2cgeoform:static/dist/styles.css'){{'}}'}}" rel="stylesheet">
    <script>
      // Register custom projection
      c2cgeoform.registerProjection(
        'EPSG:21781',
        [
          '+proj=somerc +lat_0=46.95240555555556 +lon_0=7.439583333333333 +k_0=1',
          '+x_0=600000 +y_0=200000 +ellps=bessel',
          '+towgs84=660.077,13.551,369.344,2.484,1.783,2.939,5.66 +units=m +no_defs',
        ].join(' ')
      )
    </script>
{% raw %}
    <!-- Deform JavaScript -->
    {% if deform_dependencies %}
    <script src="{{'{{'}}request.static_url('deform:static/scripts/deform.js'){{'}}'}}" type="text/css" />
    {% for js in deform_dependencies['js'] %}
    <script src="{{'{{'}}request.static_url(js){{'}}'}}"></script>
    {% endfor %}
    {% endif %}
  </head>

  <body>

    {% include 'c2cgeoform:/templates/navigation.jinja2' %}

    {% block content %}
      <p>No content</p>
    {% endblock content %}
{% endraw %}
  </body>
</html>
