@import '../node_modules/ol/ol.css';

.map {
  height: 400px;
  width: 100%;
  position: relative;
  &.hovering:hover {
    cursor: pointer;
  }
  &.cross {
    &:before,
    &:after {
      position: absolute;
      content: '';
      background: red;
      display: block;
      width: 40%;
      height: 1px;
      transform: rotate(-90deg);
      left: 0;
      right: 0;
      top: 0;
      bottom: 0;
      margin: auto;
      z-index: 100;
      max-width: 10em;
    }
    &:after {
      transform: rotate(0deg);
    }
    height: 60vw;
  }
}

.c2cgeoform-locate-me-btn {
  top: 4.5em;
  left: 0.5em;
}
.ol-touch .c2cgeoform-locate-me-btn {
  top: 5.5em;
}
.c2cgeoform-locate-me-btn .glyphicon {
  top: 0;
}
.c2cgeoform-readonly .c2cgeoform-locate-me-btn {
  display: none;
}

.c2cgeoform-controls {
  bottom: 0.5em;
  left: calc(50% - 1em);
  display: flex;
}

.ol-attribution {
  &.ol-uncollapsible {
    height: 1.5em;
  }
  ul {
    font-size: 1.2rem;
  }
}
